@@ -6,9 +6,6 @@
 #define GKO_BENCHMARK_UTILS_GENERAL_HPP_
 
 
-#include <ginkgo/ginkgo.hpp>
-
-
 #include <algorithm>
 #include <array>
 #include <fstream>
@@ -24,13 +21,12 @@
 #include <utility>
 #include <vector>
 
-
 #include <gflags/gflags.h>
 
+#include <ginkgo/ginkgo.hpp>
 
 #include <ginkgo/core/base/memory.hpp>
 
-
 #include "benchmark/utils/json.hpp"
 #include "benchmark/utils/timer.hpp"
 #include "benchmark/utils/types.hpp"
@@ -172,11 +168,11 @@ void initialize_argument_parsing(int* argc, char** argv[], std::string& header,
  *
  * @param extra  describes benchmark specific extra parameters to output
  */
-void print_general_information(const std::string& extra)
+void print_general_information(const std::string& extra,
+                               std::shared_ptr<const gko::Executor> exec)
 {
     std::clog << gko::version_info::get() << std::endl
-              << "Running on " << FLAGS_executor << "(" << FLAGS_device_id
-              << ")\n"
+              << "Running on " << exec->get_description() << std::endl
               << "Running with " << FLAGS_warmup << " warm iterations and ";
     if (FLAGS_repetitions == "auto") {
         std::clog << "adaptively determined repetititions with "
