@@ -17,10 +17,11 @@ namespace ic_factorization {
 
 
 template <typename ValueType, typename IndexType>
-void compute(std::shared_ptr<const DefaultExecutor> exec,
-             matrix::Csr<ValueType, IndexType>* m) GKO_NOT_IMPLEMENTED;
+void sparselib_ic(std::shared_ptr<const DefaultExecutor> exec,
+                  matrix::Csr<ValueType, IndexType>* m) GKO_NOT_IMPLEMENTED;
 
-GKO_INSTANTIATE_FOR_EACH_VALUE_AND_INDEX_TYPE(GKO_DECLARE_IC_COMPUTE_KERNEL);
+GKO_INSTANTIATE_FOR_EACH_VALUE_AND_INDEX_TYPE(
+    GKO_DECLARE_IC_SPARSELIB_IC_KERNEL);
 
 
 }  // namespace ic_factorization
