@@ -6,14 +6,12 @@
 #define GKO_DPCPP_MATRIX_BATCH_STRUCT_HPP_
 
 
-#include "core/matrix/batch_struct.hpp"
-
-
 #include <ginkgo/core/matrix/batch_dense.hpp>
 #include <ginkgo/core/matrix/batch_ell.hpp>
 
-
 #include "core/base/batch_struct.hpp"
+#include "core/matrix/batch_struct.hpp"
+#include "dpcpp/base/types.hpp"
 
 
 namespace gko {
@@ -35,10 +33,11 @@ namespace dpcpp {
  * Generates an immutable uniform batch struct from a batch of csr matrices.
  */
 template <typename ValueType, typename IndexType>
-inline batch::matrix::csr::uniform_batch<const ValueType, const IndexType>
+inline batch::matrix::csr::uniform_batch<const device_type<ValueType>,
+                                         const IndexType>
 get_batch_struct(const batch::matrix::Csr<ValueType, IndexType>* const op)
 {
-    return {op->get_const_values(),
+    return {as_device_type(op->get_const_values()),
             op->get_const_col_idxs(),
             op->get_const_row_ptrs(),
             op->get_num_batch_items(),
@@ -52,10 +51,10 @@ get_batch_struct(const batch::matrix::Csr<ValueType, IndexType>* const op)
  * Generates a uniform batch struct from a batch of csr matrices.
  */
 template <typename ValueType, typename IndexType>
-inline batch::matrix::csr::uniform_batch<ValueType, IndexType> get_batch_struct(
-    batch::matrix::Csr<ValueType, IndexType>* const op)
+inline batch::matrix::csr::uniform_batch<device_type<ValueType>, IndexType>
+get_batch_struct(batch::matrix::Csr<ValueType, IndexType>* const op)
 {
-    return {op->get_values(),
+    return {as_device_type(op->get_values()),
             op->get_col_idxs(),
             op->get_row_ptrs(),
             op->get_num_batch_items(),
@@ -69,10 +68,10 @@ inline batch::matrix::csr::uniform_batch<ValueType, IndexType> get_batch_struct(
  * Generates an immutable uniform batch struct from a batch of dense matrices.
  */
 template <typename ValueType>
-inline batch::matrix::dense::uniform_batch<const ValueType> get_batch_struct(
-    const batch::matrix::Dense<ValueType>* const op)
+inline batch::matrix::dense::uniform_batch<const device_type<ValueType>>
+get_batch_struct(const batch::matrix::Dense<ValueType>* const op)
 {
-    return {op->get_const_values(), op->get_num_batch_items(),
+    return {as_device_type(op->get_const_values()), op->get_num_batch_items(),
             static_cast<int32>(op->get_common_size()[1]),
             static_cast<int32>(op->get_common_size()[0]),
             static_cast<int32>(op->get_common_size()[1])};
@@ -83,10 +82,10 @@ inline batch::matrix::dense::uniform_batch<const ValueType> get_batch_struct(
  * Generates a uniform batch struct from a batch of dense matrices.
  */
 template <typename ValueType>
-inline batch::matrix::dense::uniform_batch<ValueType> get_batch_struct(
-    batch::matrix::Dense<ValueType>* const op)
+inline batch::matrix::dense::uniform_batch<device_type<ValueType>>
+get_batch_struct(batch::matrix::Dense<ValueType>* const op)
 {
-    return {op->get_values(), op->get_num_batch_items(),
+    return {as_device_type(op->get_values()), op->get_num_batch_items(),
             static_cast<int32>(op->get_common_size()[1]),
             static_cast<int32>(op->get_common_size()[0]),
             static_cast<int32>(op->get_common_size()[1])};
@@ -97,10 +96,11 @@ inline batch::matrix::dense::uniform_batch<ValueType> get_batch_struct(
  * Generates an immutable uniform batch struct from a batch of ell matrices.
  */
 template <typename ValueType, typename IndexType>
-inline batch::matrix::ell::uniform_batch<const ValueType, const IndexType>
+inline batch::matrix::ell::uniform_batch<const device_type<ValueType>,
+                                         const IndexType>
 get_batch_struct(const batch::matrix::Ell<ValueType, IndexType>* const op)
 {
-    return {op->get_const_values(),
+    return {as_device_type(op->get_const_values()),
             op->get_const_col_idxs(),
             op->get_num_batch_items(),
             static_cast<IndexType>(op->get_common_size()[0]),
@@ -114,10 +114,10 @@ get_batch_struct(const batch::matrix::Ell<ValueType, IndexType>* const op)
  * Generates a uniform batch struct from a batch of ell matrices.
  */
 template <typename ValueType, typename IndexType>
-inline batch::matrix::ell::uniform_batch<ValueType, IndexType> get_batch_struct(
-    batch::matrix::Ell<ValueType, IndexType>* const op)
+inline batch::matrix::ell::uniform_batch<device_type<ValueType>, IndexType>
+get_batch_struct(batch::matrix::Ell<ValueType, IndexType>* const op)
 {
-    return {op->get_values(),
+    return {as_device_type(op->get_values()),
             op->get_col_idxs(),
             op->get_num_batch_items(),
             static_cast<IndexType>(op->get_common_size()[0]),
