@@ -4,17 +4,16 @@
 
 #include "core/stop/residual_norm_kernels.hpp"
 
-
-#include <CL/sycl.hpp>
-
+#include <sycl/sycl.hpp>
 
 #include <ginkgo/core/base/exception_helpers.hpp>
 #include <ginkgo/core/base/math.hpp>
 #include <ginkgo/core/stop/residual_norm.hpp>
 
-
 #include "core/base/array_access.hpp"
 #include "dpcpp/base/dim3.dp.hpp"
+#include "dpcpp/base/math.hpp"
+#include "dpcpp/base/types.hpp"
 #include "dpcpp/components/thread_ids.dp.hpp"
 
 
@@ -49,7 +48,7 @@ void residual_norm(std::shared_ptr<const DpcppExecutor> exec,
     });
 
     auto orig_tau_val = orig_tau->get_const_values();
-    auto tau_val = tau->get_const_values();
+    auto tau_val = as_device_type(tau->get_const_values());
     auto stop_status_val = stop_status->get_data();
     exec->get_queue()->submit([&](sycl::handler& cgh) {
         cgh.parallel_for(
@@ -105,13 +104,13 @@ void implicit_residual_norm(
     });
 
     auto orig_tau_val = orig_tau->get_const_values();
-    auto tau_val = tau->get_const_values();
+    auto tau_val = as_device_type(tau->get_const_values());
     auto stop_status_val = stop_status->get_data();
     exec->get_queue()->submit([&](sycl::handler& cgh) {
         cgh.parallel_for(
             sycl::range<1>{tau->get_size()[1]}, [=](sycl::id<1> idx_id) {
                 const auto tidx = idx_id[0];
-                if (std::sqrt(std::abs(tau_val[tidx])) <=
+                if (gko::sqrt(gko::abs(tau_val[tidx])) <=
                     rel_residual_goal * orig_tau_val[tidx]) {
                     stop_status_val[tidx].converge(stoppingId, setFinalized);
                     device_storage_val[1] = true;
