@@ -2,17 +2,13 @@
 //
 // SPDX-License-Identifier: BSD-3-Clause
 
-#include <ginkgo/core/base/device.hpp>
-
-
-#include <hip/hip_runtime.h>
-
+#include "ginkgo/core/base/device.hpp"
 
 #include <ginkgo/config.hpp>
 #include <ginkgo/core/base/exception_helpers.hpp>
 #include <ginkgo/core/base/stream.hpp>
 
-
+#include "common/cuda_hip/base/runtime.hpp"
 #include "hip/base/scoped_device_id.hip.hpp"
 
 
