@@ -2,15 +2,11 @@
 //
 // SPDX-License-Identifier: BSD-3-Clause
 
-#include <ginkgo/core/base/memory.hpp>
-
-
-#include <hip/hip_runtime.h>
-
+#include "ginkgo/core/base/memory.hpp"
 
 #include <ginkgo/core/base/exception_helpers.hpp>
 
-
+#include "common/cuda_hip/base/runtime.hpp"
 #include "hip/base/scoped_device_id.hip.hpp"
 
 
